from configparser import ConfigParser
import os
from os.path import dirname, join
import subprocess


def get_config():
    """
    Read configuration file, perform appropriate type conversion
    and return the whole configuration.

    :return dict[dict]: configuration
    """
    config = ConfigParser()
    config.read('config.ini')

    # get requests config
    requests_config = config['REQUESTS']
    requests_params = {}
    requests_params['user_agent'] = requests_config['user_agent']
    requests_params['max_retries'] = int(requests_config['max_retries'])
    requests_params['backoff_factor'] = float(
        requests_config['backoff_factor']
    )

    requests_params['retry_on'] = tuple(
        map(int, requests_config['retry_on'].split(',')))

    requests_params['timeout'] = int(requests_config['timeout'])

    return {
        'AWS': config['AWS'],
        'REQUESTS': requests_params,
        'SCRAPING': config['SCRAPING'],
        'DATABASE': config['DATABASE']
    }


def get_environment_variables():
    """
    Read the configuration from the environment variables.

    :return dict[dict]: configuration
    """
    configuration = {}

    configuration['AWS'] = {
        's3_bucket': os.getenv('FINANCE_SCRAPING_S3_BUCKET'),
        'profile': os.getenv('FINANCE_SCRAPING_AWS_PROFILE')
    }
    # set AWS profile to None if user typed 'None'
    if configuration['AWS']['profile'] == 'None':
        configuration['AWS']['profile'] = None

    configuration['REQUESTS'] = {
        'user_agent': os.getenv('FINANCE_SCRAPING_USER_AGENT'),
        'max_retries': int(os.getenv('FINANCE_SCRAPING_MAX_RETRIES')),
        'backoff_factor': float(os.getenv('FINANCE_SCRAPING_BACKOFF_FACTOR')),
        'retry_on': tuple(
            map(int, os.getenv('FINANCE_SCRAPING_RETRY_ON').split(','))),
        'timeout': int(os.getenv('FINANCE_SCRAPING_TIMEOUT'))
    }

    configuration['SCRAPING'] = {
        'urls_s3_key': os.getenv('FINANCE_SCRAPING_URLS_S3_KEY')}

    configuration['DATABASE'] = {
        'database': os.getenv('FINANCE_SCRAPING_DATABASE'),
        'table': os.getenv('FINANCE_SCRAPING_TABLE'),
        'db_user': os.getenv('FINANCE_SCRAPING_DB_USER'),
        'password': os.getenv('FINANCE_SCRAPING_PASSWORD'),
        'host': os.getenv('FINANCE_SCRAPING_HOST'),
        'port': os.getenv('FINANCE_SCRAPING_PORT'),
    }

    return configuration


def configure():
    """
    Run BASH configuration script.
    """
    config_file_path = join(
        dirname(__file__),
        join('scripts', 'configure.sh')
    )
    try:
        subprocess.run(['bash', config_file_path])
    # ProcessLookupError happens if you hit ctrl+c
    except ProcessLookupError:
        msg = (
            "\nConfiguration may be incomplete, "
            "run 'finance-scraping -c' again."
        )
        print(msg)



import csv
from datetime import datetime
from io import BytesIO, StringIO
import logging
from os.path import dirname, join
import time
from zipfile import ZIP_DEFLATED, ZipFile

from finance_scraping import (
    aws,
    config,
    loading,
    parsing_html,
    scraping,
    utils
)

LOG_FILE_NAME = 'log.txt'
RAW_PAGES_S3_PREFIX = 'raw-page-content'
RAW_PAGES_S3_SUFFIX = 'archive.zip'
SECURITY_REPORT_S3_PREFIX = 'parsed-page-data'
SECURITY_REPORT_S3_SUFFIX = 'security_report.csv'
CSV_HEADER = [
    'company_name',
    'capital',
    'date',
    'time',
    'last_quote',
    'last_close',
    'daily_change_abs',
    'daily_change_rel',
    'bid',
    'offer',
    'low',
    'high',
    'day_volume',
    'p_e',
    'yield_percent',
    'collection_date'
]
CREATE_TABLE_SQL = """
    CREATE TABLE IF NOT EXISTS {} (
    company_name VARCHAR,
    capital FLOAT,
    date DATE,
    time TIME,
    last_quote FLOAT,
    last_close FLOAT,
    daily_change_abs FLOAT,
    daily_change_rel FLOAT,
    bid FLOAT,
    offer FLOAT,
    low FLOAT,
    high FLOAT,
    day_volume FLOAT,
    p_e FLOAT,
    yield_percent FLOAT,
    collection_date DATE
    );"""


def setup_logging(log_file):
    """
    Setup logging.
    """
    fmt = '%(asctime)s %(levelname)s: %(message)s'
    log_file_path = join(dirname(__file__), log_file)

    logging.basicConfig(
        filename=log_file_path,
        format=fmt,
        filemode='a',
        level=logging.INFO
    )


def extract():
    """
    Perform the extraction steps of webscraping:
        - download raw pages contents
        - zip data
        - upload zipped archive to AWS S3

    This function takes no parameters since all parameters are collected from
    environment variables.
    """
    # collect functions parameters from environment variables
    params = config.get_environment_variables()
    bucket = params['AWS']['s3_bucket']
    profile = params['AWS']['profile']

    # get list of urls to scrape
    urls_list = scraping.get_urls(
        bucket,
        params['SCRAPING']['urls_s3_key'],
        profile
    )

    session = scraping.get_session(
        params['REQUESTS']['max_retries'],
        params['REQUESTS']['backoff_factor'],
        params['REQUESTS']['retry_on']
    )

    # open a ZipFile archive to be stored in memory
    zip_file_object = BytesIO()
    zip_archive = ZipFile(zip_file_object, 'a', compression=ZIP_DEFLATED)

    # iterate through URL and download page content
    logging.info('scraping started')

    for url in urls_list:

        results = scraping.download_page_contents(
            session,
            url,
            params['REQUESTS']['user_agent'],
            params['REQUESTS']['timeout'],
            params['REQUESTS']['max_retries']
        )

        # if page scraping was successful
        # zip page contents as a text file in an archive
        if results is not None:
            text_buffer = StringIO(results)
            security_id = scraping.get_security_id(url)
            zip_archive.writestr(f'{security_id}.txt', text_buffer.getvalue())

        time.sleep(0.5)

    zip_archive.close()

    # upload ZIP archive containing raw page contents to S3
    date = datetime.today().strftime('%Y/%m/%d')
    archive_key = aws.get_s3_key(
        RAW_PAGES_S3_PREFIX,
        RAW_PAGES_S3_SUFFIX,
        date
    )
    msg = f"uploading pages contents to s3://{bucket}/{archive_key}"
    logging.info(msg)

    aws.upload_object_to_s3(
        zip_file_object,
        bucket,
        archive_key,
        profile
    )

    logging.info('scraping finished')


def transform(date=None):
    """
    Perform the transform step of the pipeline:
        - download and unzip raw page archive from AWS S3
        - parse page contents to retrieve financial data
        - write data as a CSV file
        - upload CSV to S3

    AWS parameters are collected from environment variables.
    If you want to transform data collected at a date other than today,
    pass the 'date' argument.

    :param str date: date corresponding to the data to process,
                     optional (default None)
    """
    if not date:
        date = datetime.today().strftime('%Y/%m/%d')
    else:
        date = utils.format_date(date)

    # collect functions parameters from environment variables
    params = config.get_environment_variables()
    bucket = params['AWS']['s3_bucket']
    profile = params['AWS']['profile']

    # download raw web pages contents
    archive_key = aws.get_s3_key(
        RAW_PAGES_S3_PREFIX,
        RAW_PAGES_S3_SUFFIX,
        date
    )

    logging.info(f'downloading raw pages {archive_key} from bucket {bucket}')

    raw_pages_zip = aws.download_s3_object(
        bucket,
        archive_key,
        profile
    )

    # we will unzip web pages contents in memory
    logging.info('reading raw pages archive')
    zip_data = BytesIO(raw_pages_zip)
    zip_obj = ZipFile(zip_data, 'r')

    # we will write data parsed from the web page to a csv in memory
    # 1 page = 1 row
    csv_obj = StringIO()
    writer = csv.DictWriter(csv_obj, CSV_HEADER, lineterminator='\n')
    writer.writeheader()

    # loop through files in the archive (1 file = 1 web page)
    for file_name in zip_obj.namelist():

        logging.info(f'parsing {file_name}')

        # extract file content, parse HTML code, write to csv
        page_contents = zip_obj.read(file_name).decode()
        parsed = parsing_html.parse_webpage(page_contents, date)
        writer.writerow(parsed)

    csv_key = aws.get_s3_key(
        SECURITY_REPORT_S3_PREFIX,
        SECURITY_REPORT_S3_SUFFIX,
        date
    )

    logging.info(f'uploading file {csv_key} to bucket {bucket}')
    aws.upload_object_to_s3(
        csv_obj,
        bucket,
        csv_key,
        profile
    )

    logging.info('transform finished')


def load(date=None):
    """
    Load the CSV file containing security data into the database.

    :param str date: date of security data collection, optional (default None)
    """
    if not date:
        date = datetime.today().strftime('%Y/%m/%d')
    else:
        date = utils.format_date(date)

    # collect functions parameters from environment variables
    params = config.get_environment_variables()
    bucket = params['AWS']['s3_bucket']
    profile = params['AWS']['profile']
    con_params = params['DATABASE']
    database = con_params['database']
    table_name = con_params['table']

    # create table if exists
    logging.info(f"creating table '{table_name}' if not exists")
    con = loading.get_connection(con_params)
    loading.execute_sqls([(CREATE_TABLE_SQL.format(table_name), ())], con)

    # to make the 'load' step idempotent, we check if data for the specified
    # date is already in the database, if it is we either stop or delete and
    # load again
    con = loading.get_connection(con_params)
    loading.check_table_for_loaded_data(table_name, date, con)

    # download CSV file to be loaded and remove header
    csv_key = aws.get_s3_key(
        SECURITY_REPORT_S3_PREFIX,
        SECURITY_REPORT_S3_SUFFIX,
        date
    )
    logging.info(f"downloading '{csv_key}' from S3 bucket '{bucket}'")
    csv_data = aws.download_s3_object(
        bucket,
        csv_key,
        profile
    )
    csv_no_header = utils.remove_header(csv_data.decode())

    # load the whole CSV at once
    logging.info(
        f"loading data into database '{database}' "
        f"in table '{table_name}'"
    )
    con = loading.get_connection(con_params)
    loading.copy_into(
        file_object=StringIO(csv_no_header),
        table_name=table_name,
        connection=con
    )
    logging.info('loading finished')


def main():
    """
    Run the entire ETL pipeline.
    """
    args = utils.parse_cli()

    if args.configure:
        config.configure()

    setup_logging(LOG_FILE_NAME)

    if args.extract:
        extract()

    if args.transform:
        transform(args.date)

    if args.load:
        load(date=args.date)


if __name__ == '__main__':
    main()



import logging
import os
import re
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
import time

import finance_scraping.aws as aws


def validate_urls(urls):
    """
    Make sure each URL in a list starts with 'http://' and is not empty,
    which can happen when reading URLs from a file.

    :param list(str) urls: list of URLs
    :return list(str): list of validated URLs
    """
    validated = []

    for u in urls:
        if u.startswith('http://'):
            validated.append(u)

        elif u.startswith('www.'):
            validated.append(f'http://{u}')

        else:
            continue

    return validated


def get_urls(bucket, urls_key, aws_profile=None):
    """
    Get the list of URLs to scrape from a text file stored in S3.

    :param str bucket: AWS S3 bucket where the file containing URLS is stored
    :param str urls_key: AWS S3 key to the file containing URLs
    :param str aws_profile: AWS profile to use (default None)
    :return list(str): list of URLs
    """
    data = aws.download_s3_object(
        bucket,
        urls_key,
        aws_profile
    )

    text = data.decode()
    urls = text.split(os.linesep)
    validated = validate_urls(urls)
    return validated


def get_session(max_retries, backoff_factor, retry_on):
    """
    Get session object for HTTP requests.

    To have granular control on request retries, we define a 'Retry' object
    from urllib3 and use it as a parameter for the HTTP adapter mounted on the
    Session object.

    :param int max_retries: maximum number of retries on HTTP request failure
    :param float backoff_factor: delay backoff factor for HTTP request retries
    :param list(int) retry_on: HTTP status codes allowing HTTP request retry
    :return requests.Session: session object
    """
    session = requests.Session()

    retry = Retry(
        total=max_retries,
        read=max_retries,
        connect=max_retries,
        backoff_factor=backoff_factor,
        status_forcelist=retry_on
    )

    adapter = HTTPAdapter(max_retries=retry)
    session.mount('http://', adapter)
    session.mount('https://', adapter)
    return session


def download_page_contents(
        session,
        url,
        user_agent,
        timeout,
        max_retries
):
    """
    Download the contents of a web page.

    If the server's response was 200 return the text part of the response.
    If the response is not 200 and not one of the status codes the Session
    object is programmed to retry on (typically 500, 502 and 504), send the
    status code.
    We catch other unexpected errors and retry a maximum of 10 times.

    :param requests.Session: Session object
    :param str url: page URL
    :param str user_agent: custome user-agent to put in request headers
    :param int timeout: allowed period for the server to respond (seconds)
    :param int max_retries: maximum number of retries on HTTP request failure
    :return str: web page content
    """
    headers = {'User-Agent': user_agent}

    while max_retries > 0:

        try:
            response = session.get(url, timeout=timeout, headers=headers)
            if response.ok:
                logging.info(f'scraped {url}')
                return response.text

            else:
                msg = f'scraping failed: {response.status_code} for {url}'
                logging.error(msg)
                return

        except Exception as e:
            msg = f'scraping failed: {e} for {url}'
            logging.error(msg)
            max_retries -= 1
            time.sleep(10)


def get_security_id(url):
    """
    Get security ID from the URL where financial information about this
    security is displayed, using a regular expression.

    :param str url: url of the company's page
    :return str: ID of the security
    """
    result = re.search('(?<=&id=).{10}', url)
    return result.group(0)



import logging
import sys

import psycopg2


def get_connection(parameters):
    """
    Connect to a database.

    :param dict parameters: dictionary of connection parameters
                            where the keys are: database, db_user,
                            password, host and port
    :return: database connection
    """
    engine = psycopg2.connect(
        database=parameters['database'],
        user=parameters['db_user'],
        password=parameters['password'],
        host=parameters['host'],
        port=parameters['port'],
    )
    return engine


def table_exists(table_name, connection, database_engine='postgres'):
    """
    Check if the table exists in the database.

    :param str table_name: name of the table
    :param connection: psycopg2 connection object
    :param str database_engine: optional, default 'postgres'
    :return bool: True if the table exists, else False
    """
    cur = connection.cursor()

    if database_engine == 'postgres':
        query = """
            SELECT EXISTS (
                SELECT 1
                FROM information_schema.tables
                WHERE table_name = %s
            );"""
        cur.execute(query, (table_name,))
        return cur.fetchone()[0]


def execute_sqls(statements, connection):
    """
    Execute a list of SQL statements.

    Statements should be passed this way:
    [("SELECT * FROM friends WHERE age > %s", (42,)), ...]

    :param list statements: a list of SQL statements and associated query
                            parameters
    :param connection: a psycopg2 connection object
    """
    cur = connection.cursor()
    try:
        for s in statements:
            cur.execute(*s)
    except Exception as e:
        logging.critical(e)
        connection.rollback()
        raise
    finally:
        connection.commit()
        connection.close()


def execute_sql_file(file_name, connection):
    with open(file_name) as f:
        sqls = tuple(
            s.strip() + ';' for s in f.read().split(';') if s != '\n'
        )
    statements = []
    for s in sqls:
        statements.append((s, ()))
    execute_sqls(statements, connection)


def data_already_loaded(table_name, collection_date, connection):
    """
    Determines if data corresponding to the collection date was already
    loaded in the database.

    :param str table_name: name of the table
    :param str collection_date: date of web scraping
    :param connection: psycopg2 connection object
    :return bool: True if rows having the specified collection date are
                  present in the database
    """
    statement = f"""
        SELECT EXISTS (
            SELECT 1 FROM {table_name}
            WHERE collection_date = %s
        );"""
    cur = connection.cursor()
    cur.execute(statement, (collection_date,))
    return cur.fetchone()[0]


def check_table_for_loaded_data(
    table_name,
    collection_date,
    connection,
    delete_if_found=False
):
    """
    Check the table for already loaded data corresponding to the collection
    date.

    :param str table_name: name of the table to check
    :param str collection_date: date when data is collected
    :param connection: psycopg2 connection object
    :param bool delete_if_found: delete already loaded data if it is found
    """
    if data_already_loaded(table_name, collection_date, connection):
        msg = f"data was already loaded on {collection_date}"
        logging.info(msg)

        if delete_if_found:
            msg = f"deleting already loaded data from table '{table_name}'"
            logging.info(msg)
            delete_statement = "DELETE FROM %s WHERE collection_date = %s;"
            execute_sqls(
                [(delete_statement, (table_name, collection_date))],
                connection
            )

        else:
            msg = f"aborting load in table '{table_name}'"
            logging.info(msg)
            sys.exit()


def copy_into(
    file_object,
    table_name,
    connection,
    separator=',',
    null_if='NULL'
):
    """
    Copy a file-like object into the specified table.

    :param str file_object: should have read() and readline() methods
    :param str table_name: name of the table to load data into
    :param connection: psycopg2 connection object
    :param str separator: field separator of the file, optional (default ',')
    :param str null_if: textual representation of NULL values in the file,
                        optional (default 'NULL')
    """
    cur = connection.cursor()
    try:
        cur.copy_from(
            file=file_object,
            table=table_name,
            sep=separator,
            null=null_if
        )
    except Exception as e:
        logging.critical(e)
        connection.rollback()
        raise
    finally:
        connection.commit()
        connection.close()



from math import isnan
import re

from bs4 import BeautifulSoup


def string_to_float(string):
    """
    Convert a string with a comma (i.e. '3.14') to a float.

    :param str string: string representing a float
    :return numpy.float:
    """
    # \xa0 is non-breaking space in latin1
    string_modif = string.replace(u',', u'.').replace(u'\xa0', u'')

    # empty strings or symbols (e.g. '-') would throw a ValueError
    try:
        return float(string_modif)
    except ValueError:
        return float('nan')


def to_null(d):
    """
    Converts the NaN (not a number) or empty values of a dictionary to the
    string 'NULL'.

    :param dict d: dictionary to convert
    :return dict: dictionary where NaN and empty values are 'NULL'
    """
    for key, value in d.items():
        if isinstance(value, float):
            if isnan(value):
                d[key] = 'NULL'
        elif isinstance(value, str):
            if not value:
                d[key] = 'NULL'
    return d


def get_company_name(soup):
    """
    :param soup: BeautifulSoup object
    :return str: company name
    """
    found = soup.find(
        'span',
        attrs={'class': 'securityName'}
    ).get_text()
    return found


def get_last_quote(soup):
    """
    :param soup: BeautifulSoup object
    :return str: last quote of the day
    """
    found = soup.find('span', attrs={'id': 'Col0Price'}).get_text()
    return string_to_float(found)


def get_date_time(soup):
    """
    :param soup: BeautifulSoup object
    :return tuple(str, str): date and time of web page update
    """
    # get date and time
    timestamp = soup.find(
        'p',
        attrs={'id': 'Col0PriceTime'}
    ).get_text().split()[1]
    time = timestamp[10:]
    date_raw = timestamp[:10]

    # if there is not date, set to 'NULL'
    date = re.sub(r'(\d+)/(\d+)/(\d+)', r'\3-\2-\1', date_raw)
    if date == date_raw:
        date = 'NULL'

    return time, date


def get_daily_change(soup):
    """
    :param soup: BeautifulSoup object
    :return tuple(float, float): absolute and daily value change
    """
    quote_detail = soup.find(
        'span',
        attrs={'id': 'Col0PriceDetail'}
    ).get_text()

    quote_detail_abs = string_to_float(quote_detail.split('|')[0])
    quote_detail_rel = quote_detail.split('|')[1].replace('%', '')
    quote_detail_rel = string_to_float(quote_detail_rel) / 100

    return quote_detail_abs, quote_detail_rel


def get_bid_offer(soup):
    """
    :param soup: BeautifulSoup object
    :return tuple(float, float): bid and offer on the security
    """
    bid_offer = soup.find(
        'td',
        attrs={'id': 'Col0BidOffer'}
    ).get_text().split(' - ')

    bid = string_to_float(bid_offer[0])
    offer = string_to_float(bid_offer[1])

    return bid, offer


def get_low_high(soup):
    """
    :param soup: BeautifulSoup object
    :return tuple(float, float): daily low and high of the security
    """
    lo_hi = soup.find(
        'td',
        attrs={'id': 'Col0LowHigh'}
    ).get_text().split(' - ')

    if len(lo_hi) == 2:
        low = string_to_float(lo_hi[0])
        high = string_to_float(lo_hi[1])
    else:
        low, high = 'NULL', 'NULL'

    return low, high


def get_daily_volume(soup):
    """
    :param soup: BeautifulSoup object
    :return float: daily volume traded
    """
    day_vol = soup.find('td', attrs={'id': 'Col0DayVolume'}).get_text()
    return string_to_float(day_vol)


def get_capital(soup):
    """
    :param soup: BeautifulSoup object
    :return float: capital of the company
    """
    capital_str = soup.find('td', attrs={'id': 'Col0MCap'}).get_text()
    if capital_str[-3:] == 'Mil':
        capital = string_to_float(capital_str[:-3]) * 10e6
    elif capital_str[-3:] == 'Bil':
        capital = string_to_float(capital_str[:-3]) * 10e9
    else:
        capital = string_to_float(capital_str)

    return capital


def get_closing_value(soup):
    """
    :param soup: BeautifulSoup object
    :return float: closing value of the security
    """
    last_close = soup.find('td', attrs={'id': 'Col0LastClose'}).get_text()
    return string_to_float(last_close)


def get_PE_ratio(soup):
    """
    :param soup: BeautifulSoup object
    :return float: P/E ratio of the security
    """
    p_e = soup.find('td', attrs={'id': 'Col0PE'}).get_text()
    return string_to_float(p_e)


def get_yield(soup):
    """
    :param soup: BeautifulSoup object
    :return float: yield of the security, in percent
    """
    y = soup.find('td', attrs={'id': 'Col0Yield'}).get_text()
    y = string_to_float(y)
    return y / 100


def parse_webpage(page_contents, collection_date):
    """
    Parse the text of the HTML page.

    :param str page_contents: HTML code of the web page
    :param str collection_date: date when data was collected
    :return dict: dictionary where keys are stock attributes and values are
                  the corresponding values parsed from the HTML
    """
    results = {'collection_date': collection_date}

    # parse each relevant element of the web page, one at a time and put
    # them in a dictionary
    soup = BeautifulSoup(page_contents, 'html.parser')

    results['company_name'] = get_company_name(soup)

    results['last_quote'] = get_last_quote(soup)

    results['time'], results['date'] = get_date_time(soup)

    absolute, relative = get_daily_change(soup)
    results['daily_change_abs'] = absolute
    results['daily_change_rel'] = relative

    results['bid'], results['offer'] = get_bid_offer(soup)

    results['low'], results['high'] = get_low_high(soup)

    results['day_volume'] = get_daily_volume(soup)

    results['capital'] = get_capital(soup)

    results['last_close'] = get_closing_value(soup)

    results['p_e'] = get_PE_ratio(soup)

    results['yield_percent'] = get_yield(soup)

    # convert all NaN to 'NULL'
    results = to_null(results)

    return results



import argparse
from dateutil import parser
import os


def remove_header(text, separator=os.linesep):
    """
    Remove the first line containing headers from text data.

    :param str text: text content of a file
    :param str separator: line separator, optional (default to the OS default
                          line separator)
    :return str: input text with first line removed
    """
    no_header = text.split(separator)[1:]
    return separator.join(no_header)


def format_date(date):
    """
    Format a string representing a date into '%Y/%m/%d'.

    :param str date: date to format
    :return str: formatted date
    """
    date = parser.parse(date)
    return date.strftime('%Y/%m/%d')


def parse_cli():
    """
    Parse command-line interface arguments.

    :return: arguments
    """
    parser = argparse.ArgumentParser('Command-line interface for web scraper')

    parser.add_argument(
        '-c',
        '--configure',
        action='store_true',
        help="configure the scraper, requires user input"
    )

    parser.add_argument(
        '-e',
        '--extract',
        action='store_true',
        help='run the extract step of the pipeline'
    )

    parser.add_argument(
        '-t',
        '--transform',
        action='store_true',
        help='run the transform step of the pipeline'
    )

    parser.add_argument(
        '-l',
        '--load',
        action='store_true',
        help='run the load step of the pipeline'
    )

    parser.add_argument(
        '-d',
        '--date',
        type=str,
        help='specify date of data collection for transform or load'
    )

    args = parser.parse_args()

    error_message = (
        "Do not specify options 'extract' and 'date' at the same time, "
        "'date' should only be specified when calling 'transform' and/or "
        "'load'."
    )
    assert not (args.extract and args.date), error_message

    return args



from airflow import DAG
from airflow.operators.python_operator import PythonOperator
from datetime import datetime, timedelta

from finance_scraping.main import extract, transform, load

default_args = {
    'owner': 'airflow',
    'depends_on_past': False,
    'start_date': datetime(2019, 9, 4),
    'email_on_failure': False,
    'email_on_retry': False,
    'retries': 1,
    'retry_delay': timedelta(minutes=20)
}

dag = DAG(
    'finance_scraping_etl',
    default_args=default_args,
    schedule_interval='0 0 * * 1-5'
)

extract_task = PythonOperator(
    task_id='extract',
    python_callable=extract,
    dag=dag
)

transform_task = PythonOperator(
    task_id='transform',
    python_callable=transform,
    dag=dag
)

load_task = PythonOperator(
    task_id='load',
    python_callable=load,
    dag=dag
)

extract_task >> transform_task >> load_task



import os
from setuptools import setup

__version__ = '0.3.7'

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.md')) as f:
    long_description = f.read()

setup(
    name='finance-scraping',
    packages=['finance_scraping'],
    entry_points={
        'console_scripts': ['finance-scraping=finance_scraping.main:main']
    },
    version=__version__,
    description='Finance Scraping ETL Pipeline',
    long_description=long_description,
    url='https://github.com/jschnab/finance-scraping',
    author='Jonathan Schnabel',
    author_email='jonathan.schnabel31@gmail.com',
    license='GNU General Public License v3.0',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Financial and Insurance Industry',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Python :: 3.6',
    ],
    python_requires='>=3.6.8',
    keywords='security web scraping etl database',
    install_requires=[
        'beautifulsoup4==4.7.1',
        'boto3==1.9.210',
        'psycopg2==2.8.3',
        'requests==2.22.0',
        'responses==0.10.6'
    ]
)



# this module stores utility functions to interact with AWS

import boto3


def get_s3_key(prefix, suffix, date):
    """
    Get an AWS S3 key formatted with the date.

    :param str prefix: prefix of the S3 key
    :param str suffix: suffix of the S3 key
    :param str date: date
    :return str: AWS S3 key
    """
    key = f"{prefix}/{date}/{suffix}"
    return key


def get_client(service, profile):
    """
    Get an AWS client.

    :param str service: AWS service, e.g. 's3'
    :param str profile: AWS profile
    :return boto3.Client: client for API calls
    """
    client = boto3.Session(profile_name=profile).client(service)
    return client


def download_s3_object(bucket, key, profile):
    """
    Download data from an AWS S3 object.

    :param str bucket: AWS S3 bucket where the object is stored
    :param str key: AWS S3 key to the object
    :para str profile: AWS profile to use
    :return bytes: data from the AWS S3 object
    """
    client = get_client('s3', profile)
    response = client.get_object(Bucket=bucket, Key=key)
    data = response.get('Body').read()
    return data


def upload_object_to_s3(file_object, bucket, key, profile):
    """
    Upload a file-like object to S3.

    :param data: file-like object
    :param str bucket: S3 bucket where to upload data
    :param str key: S3 object key where to upload data
    :para str profile: AWS profile to use
    """
    client = get_client('s3', profile)
    client.put_object(Body=file_object.getvalue(), Bucket=bucket, Key=key)
